#!/usr/bin/env python

# flake8: noqa

from __future__ import absolute_import
from nose.tools import assert_equal

import linchpin.InventoryFilters
from linchpin.InventoryFilters import InventoryProviders
